use std::collections::BTreeMap;

use crate::data::{SourceFile, Spanned, Span, Root};

use super::{Balance, LedgerEntry};

fn print_ledger_entry(root: &Root, entry: &LedgerEntry) {
    println!("{}-{:02}-{:02}: {}", entry.datestamp.0, entry.datestamp.1, entry.datestamp.2, entry.title.as_ref().map(String::as_str).unwrap_or(""));
    for bal in &entry.balances {
        let spacer = if bal.amount.is_sign_positive() { " " } else { "" };
        if Some(bal.unit.as_ref()) == root.account_spec(*bal.account).unwrap().default_unit.as_ref() {
            println!(" - {:10}: {spacer}{}", bal.account, bal.amount);
        } else {
            println!(" - {:10}: {spacer}{} {}", bal.account, bal.amount, bal.unit);
        }
    }
    // empty line afterwards
    println!("");
}

pub fn print_ledger<'l>(root: &Root, entries: impl Iterator<Item = &'l Spanned<LedgerEntry>>) {

    let mut ordering = BTreeMap::<SourceFile, BTreeMap<Span, &LedgerEntry>>::new();

    entries.for_each(|e| {
        ordering.entry(e.span().context).or_default().insert(e.span(), e.as_ref());
    });

    for (filename, entries) in ordering {
        println!("==== file {} ====", std::path::Path::new(filename.as_ref()).display());
        for (_span, le) in entries {
            print_ledger_entry(root, le);
        }
    }
}
